'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var THREE = require('three');
var OrbitControls = require('three/examples/jsm/controls/OrbitControls');
var GLTFLoader = require('three/examples/jsm/loaders/GLTFLoader');
var DRACOLoader = require('three/examples/jsm/loaders/DRACOLoader');
var dat = require('dat.gui');
var FontLoader_js = require('three/examples/jsm/loaders/FontLoader.js');
var TextGeometry_js = require('three/examples/jsm/geometries/TextGeometry.js');
var simonJsTool = require('simon-js-tool');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n["default"] = e;
  return Object.freeze(n);
}

var THREE__namespace = /*#__PURE__*/_interopNamespace(THREE);
var dat__namespace = /*#__PURE__*/_interopNamespace(dat);

function sThree(container, options) {
  let isMounted = false;
  let hasMounted = false;
  let gui;
  let scene = new THREE__namespace.Scene();
  const renderer = new THREE__namespace.WebGLRenderer();
  let dom = renderer.domElement;
  let stop;
  let fnNameMap = {
    v3: "Vector3",
    v2: "Vector2",
    v4: "Vector4",
    cc: "CubeCamera",
    oc: "OrthographicCamera",
    pc: "PerspectiveCamera",
    sc: "StereoCamera",
    bg: "BoxGeometry",
    ba: "BufferAttribute",
    bufferg: "BufferGeometry",
    capsuleg: "CapsuleGeometry",
    coneg: "ConeGeometry",
    cylinderg: "CylinderGeometry",
    dg: "DodecahedronGeometry",
    edgeg: "EdgesGeometry",
    extrudeg: "ExtrudeGeometry",
    ig: "IcosahedronGeometry",
    lg: "LatheGeometry",
    og: "OctahedronGeometry",
    pg: "PlaneGeometry",
    polyg: "PolyhedronGeometry",
    rg: "RingGeometry",
    sg: "SphereGeometry",
    tetrag: "TetrahedronGeometry",
    tg: "TorusGeometry",
    tkg: "TorusKnotGeometry",
    tubeg: "TubeGeometry",
    wfg: "WireframeGeometry",
    cg: "CircleGeometry",
    ac: "ArcCurve",
    crc3: "CatmullRomCurve3",
    cbc: "CubicBezierCurve",
    cbc3: "CubicBezierCurve3",
    ec: "EllipseCurve",
    lc: "LineCurve",
    lc3: "LineCurve3",
    qbc: "QuadraticBezierCurve",
    qbc3: "QuadraticBezierCurve3",
    splinec: "SplineCurve",
    arrowh: "ArrowHelper",
    ah: "AxesHelper",
    bh: "BoxHelper",
    b3h: "Box3Helper",
    ch: "CameraHelper",
    dlh: "DirectionalLightHelper",
    gh: "GridHelper",
    pgh: "PolarGridHelper",
    hlh: "HemisphereLightHelper",
    ph: "PlaneHelper",
    plh: "PointLightHelper",
    sh: "SkeletonHelper",
    slh: "SpotLightHelper",
    animationl: "AnimationLoader",
    audiol: "AudioLoader",
    bgl: "BufferGeometryLoader",
    cache: "Cache",
    c: "Color",
    compressedtl: "CompressedTextureLoader",
    ctl: "CubeTextureLoader",
    dtl: "DataTextureLoader",
    filel: "FileLoader",
    fl: "FontLoader",
    svgl: "SVGLoader",
    il: "ImageLoader",
    ibl: "ImageBitmapLoader",
    l: "Loader",
    lu: "LoaderUtils",
    ml: "MaterialLoader",
    ol: "ObjectLoader",
    tl: "TextureLoader",
    g: "Group",
    lbm: "LineBasicMaterial",
    ldm: "LineDashedMaterial",
    m: "Mesh",
    mbm: "MeshBasicMaterial",
    mdepthm: "MeshDepthMaterial",
    mdistancem: "MeshDistanceMaterial",
    mlm: "MeshLambertMaterial",
    mmm: "MeshMatcapMaterial",
    mnm: "MeshNormalMaterial",
    mpm: "MeshPhongMaterial",
    mphysicalm: "MeshPhysicalMaterial",
    msm: "MeshStandardMaterial",
    mtm: "MeshToonMaterial",
    p: "Points",
    pm: "PointsMaterial",
    rsm: "RawShaderMaterial",
    sm: "ShaderMaterial",
    shadowm: "ShadowMaterial",
    spritem: "SpriteMaterial",
    line: "Line",
    lp: "LineLoop",
    ls: "LineSegments",
    al: "AmbientLight",
    alp: "AmbientLightProbe",
    dl: "DirectionalLight",
    hl: "HemisphereLight",
    hlp: "HemisphereLightProbe",
    pl: "PointLight",
    ral: "RectAreaLight",
    sl: "SpotLight",
    pls: "PointLightShadow",
    dls: "DirectionalLightShadow",
    sls: "SpotLightShadow",
    lph: "LightProbeHelper",
    ralh: "RectAreaLightHelper",
    f: "Fog",
    aa: "AnimationAction",
    anc: "AnimationClip",
    am: "AnimationMixer",
    aog: "AnimationObjectGroup",
    au: "AnimationUtils",
    a: "Animation",
    anl: "AnimationLoader"
  };
  const loaderArray = [
    "animationl",
    "AnimationLoader",
    "audiol",
    "AudioLoader",
    "bgl",
    "BufferGeometryLoader",
    "compressedtl",
    "CompressedTextureLoader",
    "ctl",
    "CubeTextureLoader",
    "dtl",
    "DataTextureLoader",
    "fl",
    "FileLoader",
    "il",
    "ImageLoader",
    "ibl",
    "ImageBitmapLoader",
    "ml",
    "MaterialLoader",
    "ol",
    "ObjectLoader",
    "tl",
    "TextureLoader"
  ];
  const cacheLoader = /* @__PURE__ */ new Map();
  const gltfLoaderMap = /* @__PURE__ */ new Map();
  const dracoLoaderMap = /* @__PURE__ */ new Map();
  const animationArray = [];
  update();
  simonJsTool.addEventListener(document, "DOMContentLoaded", update);
  function destoryStop() {
    gui?.hide();
    stop?.();
    scene = null;
    renderer.dispose();
    dom = null;
    loaderArray.length = 0;
    fnNameMap = null;
    cacheLoader.clear();
    gltfLoaderMap.clear();
    dracoLoaderMap.clear();
    animationArray.length = 0;
  }
  return {
    c,
    cf,
    track,
    setUV,
    animationArray,
    glTFLoader,
    draCOLoader,
    THREE: THREE__namespace,
    scene,
    renderer,
    dom,
    setRendererAttributes
  };
  function update() {
    if (hasMounted)
      return;
    if (simonJsTool.isStr(container))
      container = document.querySelector(container) || container;
    if (!isMounted && simonJsTool.isStr(container))
      return isMounted = true;
    else if (!container)
      throw new Error(`${container} container is not found`);
    const { createCamera, createMesh, animate, mousemove, mousedown, mouseup, debug, alias, shadowType } = options;
    if (debug && !gui) {
      gui = new dat__namespace.GUI();
      gui.closed = true;
    } else {
      gui?.hide();
    }
    if (alias) {
      Object.assign(fnNameMap, alias);
      Object.keys(alias).forEach((key) => {
        if (!alias[key].includes("Loader") || loaderArray.includes(key))
          return;
        loaderArray.push(key);
      });
    }
    scene._add = function(...args) {
      scene.add(...args);
      const result = args.map((arg) => () => unmount(arg));
      return result.length === 1 ? result[0] : result;
      function unmount(arg) {
        const { material, geometry } = arg;
        material.dispose();
        geometry.dispose();
        scene.remove(arg);
      }
    };
    createMesh?.();
    const camera = createCamera?.();
    if (!camera)
      throw new Error("camera is not created");
    if (shadowType) {
      renderer.shadowMap.enabled = true;
      renderer.shadowMap.type = THREE__namespace[shadowType];
    }
    const animationOptions = {
      params: options.middleware?.({ OrbitControls: OrbitControls.OrbitControls, camera }),
      camera
    };
    if (animate) {
      const clock = new THREE__namespace.Clock();
      stop = simonJsTool.animationFrameWrapper((time) => renderer.render(scene, animate(Object.assign(animationOptions, { elapsedTime: clock.getElapsedTime(), timestamp: time })) || camera), 0);
    } else {
      simonJsTool.animationFrameWrapper(() => renderer.render(scene, camera), 0, true);
    }
    container.appendChild(dom);
    hasMounted = true;
    simonJsTool.dragEvent(dom, {
      dragStart: mousedown,
      dragMove: mousemove,
      dragEnd: mouseup
    });
    resize();
    simonJsTool.addEventListener(window, "resize", resize);
    simonJsTool.useMutationObserver(container?.parentNode, (mutations) => {
      mutations.forEach((mutation) => {
        mutation.removedNodes.forEach((node) => {
          if (node === container)
            destoryStop();
        });
      });
    }, { childList: true });
    function resize() {
      const width = container.offsetWidth;
      const height = container.offsetHeight;
      camera.aspect = Math.min(width / height, 2);
      camera.updateProjectionMatrix();
      renderer.setSize(width, height, false);
    }
  }
  function c(fnName, ...args) {
    const lowName = fnName.toLowerCase();
    const fnNameMapKey = fnNameMap[lowName];
    const _class = THREE__namespace[fnNameMapKey || fnName];
    if (!_class)
      throw new Error(`${fnName} is not found, maybe you want to use ${Object.keys(fnNameMap).filter((key) => new RegExp(fnName.split("").reduce((result, key2) => result += `${key2}(\\w+)?`, "")).test(key)).reduce((result, key) => result += `
 ${key} : ${fnNameMap[key]}`, "")} `);
    if (loaderArray.includes(lowName)) {
      if (cacheLoader.has(lowName))
        return cacheLoader.get(lowName).load(...args);
      const loader = new _class();
      cacheLoader.set(lowName, loader);
      return loader.load(...args);
    }
    return new _class(...args);
  }
  function cf(url, text, options2) {
    if (!url.endsWith(".json"))
      throw new Error("You need to use typeface.json");
    return new Promise((resolve) => new FontLoader_js.FontLoader().load(url, (font) => resolve(new TextGeometry_js.TextGeometry(text, Object.assign(options2, { font })))));
  }
  function track(...args) {
    if (!gui)
      throw new Error("gui is not created, please use debug option");
    const p = gui.domElement.parentNode;
    if (p?.childNodes.length > 1)
      p?.removeChild(p.childNodes[0]);
    if (args[0] === "color") {
      const target = args[1][args[2]];
      return gui.addColor(args[1], args[2]).onChange(() => target?.set?.(args[1][args[2]]));
    }
    return gui.add(...args);
  }
  function setUV(target, size = 2) {
    target.geometry.setAttribute("uv2", c("ba", target.geometry.attributes.uv.array, size));
  }
  function glTFLoader(url, dracoLoader, callback) {
    return new Promise((resolve) => {
      if (simonJsTool.isFn(dracoLoader)) {
        callback = dracoLoader;
        dracoLoader = void 0;
      }
      let gltfLoader;
      if (!gltfLoaderMap.get("gltf")) {
        gltfLoader = new GLTFLoader.GLTFLoader();
        gltfLoaderMap.set("gltf", gltfLoader);
      } else {
        gltfLoader = gltfLoaderMap.get("gltf");
      }
      if (dracoLoader)
        gltfLoader.setDRACOLoader(dracoLoader);
      gltfLoader.setCrossOrigin("Anonymous");
      gltfLoader.load(url, (gltf) => {
        resolve(gltf);
        callback?.(gltf);
      });
    });
  }
  function draCOLoader(decoderPath) {
    let dracoLoader;
    if (!dracoLoaderMap.get("draco")) {
      dracoLoader = new DRACOLoader.DRACOLoader();
      dracoLoaderMap.set("draco", dracoLoader);
    } else {
      dracoLoader = dracoLoaderMap.get("draco");
    }
    dracoLoader.setDecoderPath(decoderPath);
    return dracoLoader;
  }
  function setRendererAttributes(options2) {
    Object.keys(options2).forEach((key) => {
      renderer[key] = options2[key];
    });
  }
}

exports.sThree = sThree;
